precision lowp float;

uniform lowp sampler2D tex;
uniform lowp sampler2D texLightmap;
uniform lowp samplerCube texCubeMap;
uniform lowp sampler2D texSpecular;

varying mediump vec2 v_v2TexCoord;
varying mediump vec2 v_v2LightTexCoord;
varying lowp vec3 v_v3Normal;
varying mediump vec3 v_v3WorldPosition;
varying mediump vec4 v_v4Color;

uniform lowp float u_fRewindEffect;

uniform lowp float u_fBrightness;
uniform mediump vec3 u_v3EyePosition;

void main() 
{
	// red - spec
	// green - damage mask 1
	// blue - damage mask 2

	lowp vec4 v4Light = texture2D(texLightmap, v_v2LightTexCoord);
	lowp vec4 v4Spec = texture2D(texSpecular, v_v2TexCoord);
	lowp vec4 v4Texture = texture2D(tex, v_v2TexCoord);

	lowp vec4 v4Damage1 = vec4(v4Texture.rgb * v4Spec.g, 1.0); 
	lowp vec4 v4Damage2 = vec4(v4Texture.rgb * v4Spec.b, 1.0); 
	
	v4Texture = ((v4Texture * (1.0 - v_v4Color.g)) + (v4Damage1 * v_v4Color.g));
	v4Texture = ((v4Texture * (1.0 - v_v4Color.b)) + (v4Damage2 * v_v4Color.b));
	v4Texture *= v4Light;
	
	lowp float fSpecular = (v4Spec.r * (1.0 - v_v4Color.g)) + (v4Spec.g * v_v4Color.g);	
	fSpecular = (v4Spec.r * (1.0 - v_v4Color.b)) + (v4Spec.b * v_v4Color.b);	
	fSpecular *= ((v4Light.r + v4Light.g + v4Light.b) * 0.333);	
	
	mediump vec3 v3Incident = v_v3WorldPosition - u_v3EyePosition;
	mediump vec3 v3Reflection = v3Incident - 2.0 * dot(v3Incident, v_v3Normal) * v_v3Normal / dot(v_v3Normal, v_v3Normal);	
	lowp vec4 v4TextureCube = textureCube(texCubeMap, v3Reflection);
		
	lowp vec4 v4Tmp1 = (v4Texture + v4TextureCube * fSpecular) * (1.0 - u_fRewindEffect) + v4Texture.b * u_fRewindEffect;	

	lowp vec4 v4Tmp2 = 1.0 - v4Tmp1;
	gl_FragColor = v4Tmp1 * (2.0 - u_fBrightness) + (1.0 - v4Tmp2 * v4Tmp2 * v4Tmp2) * (u_fBrightness - 1.0);	
}


